#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){
	
	ofSoundStreamSetup(0,2,this, 44100, BUFFER_SIZE, 4);	
	
	left = new float[BUFFER_SIZE];
	right = new float[BUFFER_SIZE];
	
	ofBackground(0,0,0);
	
	FFTanalyzer.setup(44100, BUFFER_SIZE/2, 1);
	
	FFTanalyzer.peakHoldTime = 15; // hold longer
	FFTanalyzer.peakDecayRate = 0.95f; // decay slower
	FFTanalyzer.linearEQIntercept = 0.9f; // reduced gain at lowest frequency
	FFTanalyzer.linearEQSlope = 0.01f; // increasing gain at higher frequencies
	
	ofSetFrameRate(60);
	
	panel.setup("control", 770, 0, 300, 150);
	panel.addPanel("fft settings", 1, false);
	
	panel.setWhichPanel("fft settings");
	
	panel.addSlider("Number of Sub Octaves","NUM_OCT",	1, 1,12, false);
	
	numOctaves=1;
}

//--------------------------------------------------------------
void testApp::update(){
	panel.update();
	if(numOctaves != panel.getValueI("NUM_OCT")){
		numOctaves = panel.getValueI("NUM_OCT");
		panel.setValueI("NUM_OCT", numOctaves,0);
		FFTanalyzer.setup(44100, BUFFER_SIZE/2, numOctaves);
	}
}

//--------------------------------------------------------------
void testApp::draw(){
	
	panel.draw();
	
	static int index=0;
	float avg_power = 0.0f;	
	
	/* do the FFT	*/
	myfft.powerSpectrum(0,(int)BUFFER_SIZE/2, left,BUFFER_SIZE,&magnitude[0],&phase[0],&power[0],&avg_power);
	
	
	for (int i = 0; i < (int)(BUFFER_SIZE/2); i++){
		ofRect(10+(i*5),400, 5,-magnitude[i] * 6);
	}
	
	for (int i = 0; i < (int)(BUFFER_SIZE/2); i++){
		freq[i] = magnitude[i];
	}
	
	FFTanalyzer.calculate(freq);
	
	ofSetColor(0xffffff);
	
	float binDrawWidth = (ofGetWidth()-20)/FFTanalyzer.nAverages;
	
	for (int i = 0; i < FFTanalyzer.nAverages; i++){
		ofRect(10+(i*binDrawWidth),600, binDrawWidth-2,-FFTanalyzer.averages[i] * 6);
	}
	
	ofSetColor(0xff0000);
	for (int i = 0; i < FFTanalyzer.nAverages; i++){
		ofRect(10+(i*binDrawWidth),600-FFTanalyzer.peaks[i] * 6,binDrawWidth-2,-4);
	}
}

//--------------------------------------------------------------
void testApp::keyPressed(int key){

}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	panel.mouseDragged(x,y,button);
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	panel.mousePressed(x,y,button);
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	panel.mouseReleased();
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){
	
}
//--------------------------------------------------------------
void testApp::audioReceived 	(float * input, int bufferSize, int nChannels){	
	// samples are "interleaved"
	for (int i = 0; i < bufferSize; i++){
		left[i] = input[i*2];
		right[i] = input[i*2+1];
	}
}

